#!perl

# Takes an HTML generated by javadoc, and turns it into a .java file
# suitable for our import summaries facility.  It even does a halfway
# decent job of creating a package import list...  (It takes note of
# any reference to classes outside the current package.  It still
# can't handle the case where the current package references two
# classes of the same name in different packages.)
#
# Usage:
#   perl reverse-javadoc.perl *.html
#
# This creates one .java file for each package that's documented.


# Remove hyperlinks from the line.  Also, keeps a list of packages
# we'll have to import (javadoc uses fully qualified names inside
# hyperlinks and almost nowhere else...)
sub remove_hrefs {
    local ($line) = @_;
    local ($temp) = $line;
    while ($temp =~ m|<a href=\"([^\"]*)\.[^\.]*\.html[^\"]*\">(.*)|) {
	$temp = $2;
	$import = $1;
	if ($import ne $package && $import ne "java.lang") {
	    $imports{$package . "/" . $import} = 1;
	}
    }
    $line =~ s|<a href=\"([^\"])*\">||g;
    $line =~ s|</a>||g;
    return $line;
}

# Handle the start of a new .html file by figuring out what package
# this class is from.
sub read_file_header {
    $first_dl = 0;
    if ($first_class) {
	$first_class = 0;
    } else {
	$summaries{$package} .= "}\n\n";
    }
    $_ = <>;
    /^  (Class|Interface) (.*)\.[^.]+$/;
    $package = $2;
}

# Get the class header.
sub read_class_header {
    while (($_ = <>) && !m|</dl>|) {
	s|  <dt> ||;
	s|</?b>||g;
	$_ = remove_hrefs($_);
	if (/^extends / || /^implements /) { 
	    $summaries{$package} .= "    "; 
	}
	$summaries{$package} .= $_;
    }
    $summaries{$package} .= "{\n";
    $first_dl = 1; # first time <dl> is seen
}

# Get one method, field, or constructor.
sub read_one_component {
    local $has_parens = 0;
    while (($_ = <>) && !m|</pre>|) {
	$_ = remove_hrefs($_);
	$summaries{$package} .= $_;
	$has_parens = $has_parens || m|\(|;
    }
    chop($summaries{$package});
    $summaries{$package} .= ($has_parens) ? " {}\n" : ";\n";
}

# Create output for a single package.
sub output_package {
    local ($package) = @_;
    print "Writing out $package\n";
    $filename = $package;
    $filename =~ tr/./-/;
    open(FILE, ">$filename.java");
    print FILE "package $package;\n";
    foreach $import (keys %imports) {
	$import =~ m|(.*)/(.*)|;
	if ($1 eq $package) {
	    print FILE "import $2.*;\n";
	}
    }
    print FILE "\n";
    print FILE $summaries{$package};
    close(FILE);
}    

# Main loop
$first_class = 1;
while (<>) {
    if (m|<title>|) {
	read_file_header();
    } elsif (!$first_dl && m|<dl>|) {
	read_class_header();
    } elsif (m|<pre>| && $prev_was_aname) {
	read_one_component();
    }
    $prev_was_aname = m|^<a name=| || ($prev_was_aname && m|^<b>.*</b>$|);
}

$summaries{$package} .= "}\n";

foreach $package (keys %summaries) {
    output_package($package);
}
