#!perl

# Creates a nice, clean release, ready for distribution.

$usage 
    = "\nUsage: \n\n"
    . "    make-release [-no-sources] [-no-tag] version\n\n"
    . "where version uses only underbars, no periods or dashes.\n"
    . "\n"
    . "Example: 'make-release 1_1_4'\n"
    . "This creates a directory sheets-1_1_4, archives it in \n"
    . "sheets-1_1_4.tar.gz, and creates the CVS tag release-1_1_4.\n"
    . "\n"
    . "Use -no-tag if you don't want CVS to tag the release.\n"
    . "Use -no-sources if you don't want the distribution to include any\n"
    . "of the Sheets source code.\n";

$copy = "xcopy /q";  # /q means "quiet".  Also, "copy" doesn't
                     # seem to work right, so we use xcopy instead.
$xcopy = "xcopy /q /s"; # /s means "subdirectories also"
$make = "nmake /c /s /nologo"; # options mean, "quietly"
$pkzip = "pkzip25 -add -directories "; # add all listed files, recursively with path names.

sub process_command_line {
    $#ARGV >= 0 || die $usage;
    while ($#ARGV >= 0 && $ARGV[0] =~ /^-/) { # while first arg starts with -
	if ($ARGV[0] eq "-no-sources") {
	    $no_sources = 1;
	    shift(@ARGV);
	} elsif ($ARGV[0] eq "-no-tag") {
	    $no_tag = 1;
	    shift(@ARGV);
	}
    }
    $#ARGV >= 0 || die $usage;
    $version = $ARGV[0];

    # $dirname is a temporary dir where we copy files we're going to
    # include in the distribution.
    $dirname = "sheets";
}

sub tag_files {
    print "Tagging files\n";
    system("cvs tag release-$version");
}

sub compile_sheets {
    print "Recompiling Sheets\n";
    system("$make recompile");
}


# Looks for the file in the path (usually CLASSPATH or PATH).  If processing
# a CLASSPATH, it won't look inside .zip or .jar files.
sub find_file {
    local($file, $path) = @_;
    @dirs = split(/;/, $path);
  DIR:
    foreach $dir (@dirs) {
	if ($dir =~ /\.zip$/ || $dir =~ /\.jar$/) {
	    next DIR;
	}
	opendir(DIR, $dir) || die("Bad dir in path: $dir");
	local(@dir_contents) = readdir(DIR);
	closedir(DIR);
	foreach $entry (@dir_contents) {
	    if ($file eq $entry) {
		return($dir . "\\" . $file);
	    }
	}
    }
    die "Couldn't find $file in $path";
}

sub make_jar_file {
    print "Creating a JAR file\n";
    system("mkdir jar-file-$version");
    system("mkdir jar-file-$version\\gwydion");
    system("$xcopy gwydion\\*.class jar-file-$version\\gwydion");
    # $pat_dir = find_file("pat", $ENV{"CLASSPATH"});
    # print "Found pat in $pat_dir\n";
    # system("mkdir jar-file-$version\\pat");
    # system("$xcopy $pat_dir\\*.class jar-file-$version\\pat");
    chdir("jar-file-$version");
    system("jar -cf ..\\$dirname\\sheets.jar .");
    chdir("..");
}

sub copy_files_for_tarring {
    system("mkdir $dirname");
    print "Copying files for distribution\n";
    copy_executable_files();
    system("$copy .sheetsrc $dirname");
    system("$copy *.sheetsrc $dirname");
    system("$copy default.sheetsrc $dirname\\.sheetsrc");
    copy_doc_files();
    copy_bitmap_files();
#    copy_summary_files();
    if (!$no_sources) {
	copy_source_files();
    }
}

sub copy_executable_files {
    print("    Copying executables\n");
    system("$copy sheets.bat $dirname");
    system("$copy util\\wrapper.exe $dirname");
    system("$copy beep.au $dirname");  # not an executable, but what the hell
}

sub copy_doc_files {
    print("    Copying docs\n");
    system("$copy README $dirname");
    system("$copy CHANGES $dirname");
    system("mkdir $dirname\\doc");
    system("$copy doc\\*.html $dirname\\doc");
    system("$copy doc\\*.gif $dirname\\doc");
    system("$copy doc\\*.doc $dirname\\doc");
    system("$copy ToDo.txt $dirname\\doc");
}

sub copy_bitmap_files {
    print("    Copying bitmaps\n");
    system("mkdir $dirname\\bitmaps");
    system("$copy bitmaps\\*.bmp $dirname\\bitmaps");
}

sub copy_summary_files {
#    print("    Copying Java summaries\n");
#    system("$copy java-awt.java $dirname");
#    system("$copy java-awt-datatransfer.java $dirname");
#    system("$copy java-io.java $dirname");
#    system("$copy java-lang.java $dirname");
#    system("$copy java-util.java  $dirname");
#    system("$copy JavaApplet.java $dirname");
#    system("$copy pat.java $dirname");
}

# Source code.  Right now, we never ship *all* of our source code
# -- too much of a pain.  But we do ship our .sheets file unless
# -no-sources is specified.  This way, people can get an idea of
# how to organize a hypercode program.
#
# If we ever decide to do a true source distribution, we'd also
# need to ship wrapper.c, Java.jjt, and massage-parser.perl.
sub copy_source_files {
    print("    Copying source code\n");
    system("$copy *.sheets $dirname");
    system("$copy Sundial.html $dirname");
    system("$copy index.html $dirname");
    system("$copy gwydiontitle.gif $dirname");
    system("$copy Makefile $dirname");
    system("$copy make-release.perl $dirname");
    system("$copy *.jpg $dirname");
## New and untried
    system("mkdir $dirname\\gwydion");
    system("mkdir $dirname\\gwydion\\javaStuff");
    system("$copy gwydion\\javaStuff\\*.jjt $dirname\\gwydion\\javaStuff");
    system("$copy gwydion\\javaStuff\\*.perl $dirname\\gwydion\\javaStuff");
    system("$mkdir $dirname\\util");
    system("$copy util\\*.bat $dirname\\util");
    system("$copy util\\*.c $dirname\\util");
    system("$copy util\\*.perl $dirname\\util");
    system("mkdir $dirname\\gwydion\\html");
    system("$copy gwydion\\html\\*.jjt $dirname\\gwydion\\html");
}

sub tar_and_gzip {
    print "Running tar\n";
    # If you use a backslash on the next line, it'll do absolutely nothing
    system("tar -cf sheets-$version.tar $dirname/*");
    print "Running gzip\n";
    system("gzip sheets-$version.tar");
}

sub zip {
    print "Zipping up the distribution.\n";
    system("del sheets-$version.zip");
    system("$pkzip sheets-$version.zip $dirname/*");
}

#########################################################
# Main thread of control
process_command_line();
if (! $no_tag) {
    tag_files();
}
# compile_sheets();
make_jar_file();
copy_files_for_tarring();
tar_and_gzip();
zip();

